/*=============================================================================
Vitalnix User Management Suite
cgi/cquota.c - Quota overview
  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2005
  -- License restrictions apply (LGPL v2)

  This file is part of Vitalnix.
  Vitalnix is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published
  by the Free Software Foundation; however ONLY version 2 of the License.

  Vitalnix is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the LICENSE file
=============================================================================*/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libHX.h>
#include "cgi/base.h"
#include "cgi/sh_auth.h"

static const char *WrongUP =
  "<p class=\"red\"><b>Benutzername und/oder Kennwort falsch!</b></p>";

static void do_quota(const char *);
static int illegal_chars(const char *);
static void header(void);
static void form(const char *);
static void footer(void);

//-----------------------------------------------------------------------------
int main(int argc, const char **argv) {
    struct HXbtree *data = cgi_split(cgi_read_data(argc, argv));
    char *user = zstr(HXbtree_get(data, "user")),
         *pw   = zstr(HXbtree_get(data, "pw"));

    header();

    if(strEQ(user, "") && strEQ(pw, "")) {
        form(user);
    } else if(illegal_chars(user)) {
        printf("%s", WrongUP);
    } else if(authenticate(user, pw)) {
        do_quota(user);
    } else {
        printf("%s", WrongUP);
    }

    footer();
    printf("\n");
    return EXIT_SUCCESS;
}

static void do_quota(const char *user) {
    uid_t saved = getuid();
    char buf[256];
    FILE *fp;

    snprintf(buf, sizeof(buf),
      "/usr/local/bin/pquota -v \"%s\" | grep ^/dev/hdg1", user);
    setuid(0);
    fp = popen(buf, "r");
    setuid(saved);
    if(fp == NULL) { return; }

    printf(
      "<table cellpadding=\"2\" cellspacing=\"0\" border=\"1\""
      " class=\"bordered\"><tr>"
      "<td align=\"center\" class=\"gray\" rowspan=\"2\">Device</td>"
      "<td align=\"center\" class=\"gray\" colspan=\"5\">Blocks</td>"
      "<td align=\"center\" class=\"gray\" colspan=\"5\">Files</td>"
      "</tr><tr>"
    );
    {int i;
    for(i = 0; i < 2; ++i)
    printf(
      "<td align=\"right\" class=\"gray\">Current Use</td>"
      "<td align=\"right\" class=\"gray\">%%</td>"
      "<td align=\"right\" class=\"gray\">Soft Limit</td>"
      "<td align=\"right\" class=\"gray\">Hard Limit</td>"
      "<td align=\"right\" class=\"gray\">Grace Time</td>"
    );
    }
    printf("</tr>");

    while(fgets(buf, sizeof(buf), fp) != NULL) {
        size_t count = 0;
        char **field = HX_split(buf, " ", &count);
        char *bover = "", *de_bover = "", *iover = "", *de_iover = "";
        unsigned long long
         dqb_used = strtoull(field[1], NULL, 0),
         dqb_soft = strtoull(field[3], NULL, 0),
         dqb_hard = strtoull(field[4], NULL, 0),
         dqi_used = strtoull(field[6], NULL, 0),
         dqi_soft = strtoull(field[8], NULL, 0),
         dqi_hard = strtoull(field[9], NULL, 0);

        if(*field[2] == '*') { bover = "<b>"; de_bover = "</b>"; }
        if(*field[6] == '*') { iover = "<b>"; de_iover = "</b>"; }

        printf("<tr><td><tt>%s</tt></td>", field[0]);

        // Blocks
        printf("<td align=\"right\">%s%llu%s</td>",
         bover, dqb_used, de_bover);
        if(dqb_soft == 0) {
            printf("<td align=\"right\">-</td>");
        } else {
            printf("<td align=\"right\">%s%d%%%s</td>",
             bover, (int)(dqb_used * 100 / dqb_soft), de_bover);
        }
        printf("<td align=\"right\">%llu</td>", dqb_soft);
        printf("<td align=\"right\">%llu</td>", dqb_hard);
        printf("<td align=\"right\">%s%s%s</td>", bover, field[5], de_bover);

        // Files
        printf("<td align=\"right\">%s%llu%s</td>",
         iover, dqi_used, de_iover);
        if(dqi_soft == 0) {
            printf("<td align=\"right\">-</td>");
        } else {
            printf("<td align=\"right\">%s%d%%%s</td>",
             iover, (int)(dqi_used * 100 / dqi_soft), de_iover);
        }
        printf("<td align=\"right\">%llu</td>", dqi_soft);
        printf("<td align=\"right\">%llu</td>", dqi_hard);
        printf("<td align=\"right\">%s%s%s</td>", iover, field[9], de_iover);
        printf("</tr>");

        // Free it
        int i;
        for(i = count - 1; i >= 0; --i) {
            free(field[i]);
        }
        free(field);
    }
    printf("</table>");
    pclose(fp);
    return;
}

static int illegal_chars(const char *str) {
    while(*str != '\0') {
        if(*str == '"' || *str == '\'') { return 1; }
        ++str;
    }
    return 0;
}

//-----------------------------------------------------------------------------
static void header(void) {
    printf(
      "Content-Type: text/html\n\n"
      "<html>"
      "<head>"
      "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">"
      "<link rel=\"stylesheet\" href=\"/intranet.css\" type=\"text/css\">"
      "</head>"

      "<body>"

      "<div align=\"center\">"
      "<p>&nbsp;</p>"
      "<p>&nbsp;</p>"
      "<p style=\"font-size: 16pt;\"><b>Quota-Status abfragen</b></p>"
    );
    return;
}

static void form(const char *user) {
    printf(
      "<p>&nbsp;</p>"
      "<form method=\"POST\">"
      "<table style=\"border: 1px dotted #CCCCCC; padding-left: 40px;"
      " padding-right: 40px;\">"
        "<tr>"
          "<td align=\"right\">Benutzername</td>"
          "<td>&nbsp;</td>"
          "<td><input type=\"text\" name=\"user\" size=\"10\""
           " value=\"%s\" /></td>"
        "</tr>"
        "<tr>"
          "<td align=\"right\">Passwort</td>"
          "<td>&nbsp;</td>"
          "<td><input type=\"password\" name=\"pw\" size=\"10\" /></td>"
        "</tr>"
        "<tr>"
          "<td align=\"center\" colspan=\"3\"><input type=\"submit\""
           " style=\"padding: 0px;\" /></td>"
        "</tr>"
      "</table>"
      "</form>"
      ,
      user
    );
    return;
}

static void footer(void) {
    printf(
      "</div>"
      "</body>"
      "</html>"
    );
    return;
}

//==[ End of file ]============================================================
