/*=============================================================================
Vitalnix User Management Suite
sysprog/shared.c - Shared functions for the ACCDB system utilities
  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2005
  -- License restrictions apply (LGPL v2)

  This file is part of Vitalnix.
  Vitalnix is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published
  by the Free Software Foundation; however ONLY version 2 of the License.

  Vitalnix is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the LICENSE file
=============================================================================*/
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <libHX.h>
#include "sysprog/shared.h"

const char *Conf_paths[] = {"sysprog/../vetc", "/etc", NULL};

//-----------------------------------------------------------------------------
int SH_count_args(const char **args) {
    int argk = 0;
    while(*args++ != NULL) { ++argk; }
    return argk;
}

void SH_interactive(struct itab *qtabp) {
    char buf[MAXLNLEN];

    while(qtabp->fmt != NULL) {
        if(qtabp->type == 's') {
            char **ptr = qtabp->ptr;
            printf(qtabp->fmt, (*ptr != NULL) ? *ptr : "");
            fflush(stdout);
            fgets(buf, MAXLNLEN, stdin);
            HX_chomp(buf);
            if(*buf != '\0') { HX_strclone(ptr, buf); }
        } else if(qtabp->type == 'd') {
            int *ptr = qtabp->ptr;
            printf(qtabp->fmt, (ptr != NULL) ? *ptr : 0);
            fflush(stdout);
            fgets(buf, MAXLNLEN, stdin);
            HX_chomp(buf);
            if(*buf != '\0') { *ptr = strtol(buf, NULL, 0); }
        }

        if(!(qtabp->callback != NULL &&
         !qtabp->callback(qtabp->fmt, qtabp->type, qtabp->ptr))) {
            ++qtabp;
        }
    }

    return;
}

int SH_runcmd(const char *fmt, struct HX_repmap *map) {
    char cmd[MAXLNLEN];
    HX_strrep(cmd, MAXLNLEN, fmt, map);
    return system(cmd);
}

long SH_string_to_iday(char *s) {
    struct tm td;
    time_t r;
    memset(&td, 0, sizeof(struct tm));

    if(strchr(s, '.') != NULL) {
        // DD.MM.YYYY
        sscanf(s, "%d.%d.%d", &td.tm_mday, &td.tm_mon, &td.tm_year);
    } else if(strchr(s, '/') != NULL) {
        // MM/DD/YYYY
        sscanf(s, "%d/%d/%d", &td.tm_mon, &td.tm_mday, &td.tm_year);
    } else if(strchr(s, '-') != NULL) {
        // YYYY-MM-DD (ISO 8601) format
        sscanf(s, "%d-%d-%d", &td.tm_year, &td.tm_mon, &td.tm_mday);
    } else {
        return -1;
    }

    r = mktime(&td);
    if(r == -1) { return -1; }
    return (unsigned long)r / 86400;
}

int SH_only_digits(char *p) {
    if(p == NULL) { return 0; }
    while(*p != '\0') {
        if(!isdigit(*p++)) { return 0; }
    }
    return 1;
}

//==[ End of file ]============================================================
